--   Creates the pt_callback package and the pt_util package for
--   project-tracker. This package hooks project-tracker into
--   workflow. pt_callback requires pt_util
--
--   @file-name project-tracker/sql/pt-phase-package.sql
--
--   @author Nobuko Asakai nasakai@arsdigita.com
--  
--   @cvs-id $Id: pt-callback-package.sql,v 1.3 2001/04/19 15:34:23 nasakai Exp $
--   @see pt-task-package pt-workflow-create


create or replace package pt_util
is
    -- Same as ttracker_util.workflow_url
    function workflow_url (
        absolute_p	      in char default 't'
    ) return varchar2;

    -- Returns the project tracker url for a given task
    -- I'm not 100% sure this is the right place for this function
    function pt_url (
	task_id	    in pt_tasks.task_id%TYPE
    ) return varchar2;

    function notification_sender (
        package_id	      in apm_packages.package_id%TYPE
    ) return parties.party_id%TYPE;

end pt_util;
/
show errors

create or replace package body pt_util
is
    function workflow_url (
        absolute_p	      in char default 't'
    ) return varchar2
    is
        v_system_url    apm_parameter_values.attr_value%TYPE;
        cursor node_cur is
            select sn.node_id
              from site_nodes sn,
                   apm_packages ap
             where ap.package_key = 'acs-workflow'
               and sn.object_id = ap.package_id;
        node_rec        node_cur%ROWTYPE;
    begin
        -- try to get the id of the site node that contains acs workflow
        open node_cur;
        fetch node_cur into node_rec;
        if node_cur%NOTFOUND then
            close node_cur;
            raise_application_error(-20000, 'ACS Workflow must be mounted');
        end if;
        close node_cur;
	
	-- if only relative url is needed
	if absolute_p <> 't' then
	    return site_node.url(node_rec.node_id);
        end if;

        -- since there's only 1 instance of the kernel
        -- it's ok to use select.. into..
        select apv.attr_value into v_system_url
          from apm_parameter_values apv,
               apm_parameters ap
         where ap.package_key = 'acs-kernel'
           and ap.parameter_name = 'SystemURL'
           and ap.parameter_id = apv.parameter_id;
        
        return v_system_url||site_node.url(node_rec.node_id);

    end workflow_url;                

    function pt_url (
	task_id	    in pt_tasks.task_id%TYPE
    ) return varchar2
    is
        v_system_url    apm_parameter_values.attr_value%TYPE;
        cursor node_cur is
            select sn.node_id
              from site_nodes sn
               where sn.object_id = pt_project.get_package(task_id);
        node_rec        node_cur%ROWTYPE;
    begin
        open node_cur;
        fetch node_cur into node_rec;
        if node_cur%NOTFOUND then
            close node_cur;
            raise_application_error(-20000, 'The project is not mounted');
        end if;
        close node_cur;

        select apv.attr_value into v_system_url
          from apm_parameter_values apv,
               apm_parameters ap
         where ap.package_key = 'acs-kernel'
           and ap.parameter_name = 'SystemURL'
           and ap.parameter_id = apv.parameter_id;
        
        return v_system_url||site_node.url(node_rec.node_id);
    
    end pt_url;

    function notification_sender (
        package_id	      in apm_packages.package_id%TYPE
    ) return parties.party_id%TYPE
    is
        cursor value_cur is
            select apv.attr_value
            from apm_parameter_values apv,
                 apm_parameters ap
           where apv.package_id = notification_sender.package_id
             and ap.package_key = 'project-tracker'
             and ap.parameter_name = 'PTNotificationSender'
	     and ap.parameter_id = apv.value_id;
       value_rec  value_cur%ROWTYPE;
        v_result   parties.party_id%TYPE;
	
    begin
        open value_cur;
        fetch value_cur into value_rec;
        if value_cur%NOTFOUND then
            v_result := -1;
        else
            v_result := to_number(value_rec.attr_value);
        end if;
        close value_cur;                       
 
        return v_result;

        -- in case somebody tempered with the parameter
        exception
            when others then
                return -1;
    end notification_sender;
end pt_util;
/
show errors

create or replace package pt_callback 
as

    -- We used to have get deadline and copy attributes, but it
    -- has been decided that we do not like storing duplicate
    -- information, however, a new approach is needed if we decide
    -- that we want to send late deadline notification

    procedure complete_fire (
        case_id          in number,
        transition_key   in varchar2,
        custom_arg       in varchar2
    );

    procedure notification (
        task_id 	in number, 
        custom_arg 	in varchar2, 
        party_to 	in number,
        party_from 	in out number,
        subject 	in out varchar2, 
        body	 	in out varchar2
    );               

end pt_callback;
/
show errors

create or replace package body pt_callback 
as

    procedure notification (
        task_id 	in number, 
        custom_arg 	in varchar2, 
        party_to 	in number,
        party_from 	in out number,
        subject 	in out varchar2, 
        body	 	in out varchar2
    )
    is
	v_project_name pt_tasks.name%TYPE;

        cursor pt_task_cur
	is 
	select pt.name, pt.description,  pt.task_id, pt.estimated_days, 
	pt.deadline
	from pt_tasks pt, wf_cases wc, wf_tasks wt
	where 	wt.task_id = notification.task_id
	and wt.case_id = wc.case_id
	and wc.object_id = pt.task_id;
	
	pt_task_row pt_task_cur%ROWTYPE;
	
	v_package_id acs_objects.object_id%TYPE;
    begin
        open pt_task_cur;
        fetch pt_task_cur into pt_task_row;
        if pt_task_cur%NOTFOUND then
            close pt_task_cur;
            raise_application_error(-20000, 'Invalid task id');
        end if;

	v_project_name := pt_project.name (pt_task_row.task_id);
	
	v_package_id := pt_project.get_package(pt_task_row.task_id);

	
	party_from := pt_util.notification_sender(v_package_id);

	-- We don't have a way for overriding the notification sender
	-- with a dummy project-tracker@system.com type address.

	subject := 'Project ' || v_project_name ||': ' || pt_task_row.name;
	body := 'You have been assigned to complete the following task.

Project: ' || v_project_name || '
Task :' ||  pt_task_row.name || '
Description: ' || pt_task_row.description || ' 
Estimated days: ' || pt_task_row.estimated_days || '
Deadline: ' 	|| pt_task_row.deadline || '

manage via: ' || pt_util.workflow_url || '
project plan: ' || pt_util.pt_url(pt_task_row.task_id);

	  --  Manage via url and pt url
    end notification;                                                 

    procedure complete_fire (
        case_id          in number,
        transition_key   in varchar2,
        custom_arg       in varchar2
    )
    is
	v_task_id pt_tasks.task_id%TYPE;
	v_actual_days number;
    begin
        select object_id into v_task_id
	from wf_cases
	where case_id = complete_fire.case_id;

	-- We also copy the actual time information over.
	v_actual_days := workflow_case.get_attribute_value (
	    case_id => complete_fire.case_id,
	    attribute_name => 'actual_days'
	);		      

	update pt_tasks
	set actual_days = v_actual_days
	where task_id = v_task_id;
    end complete_fire;

end pt_callback;
/
show errors

