/*

  Creates the task package for project tracker tasks.  
  Unit test in  tests/pt-task-test.sql, requires utPL/SQL

  @file-name project-tracker/sql/pt-task-package.sql

  @author Nobuko Asakai nasakai@arsdigita.com
  
  @cvs-id $Id: pt-task-package.sql,v 1.2 2001/04/17 20:19:06 nasakai Exp $
  
*/

set serveroutput off;

create or replace package pt_task
as
    function name (
        object_id	in acs_objects.object_id%TYPE
    ) return pt_tasks.name%TYPE;

    function new (
	task_id		in pt_tasks.task_id%TYPE default null,
	parent_id       in pt_tasks.task_id%TYPE,
	name		in pt_tasks.name%TYPE,
	description	in pt_tasks.description%TYPE,
	estimated_days	in pt_tasks.estimated_days%TYPE default 1,
	deadline	in pt_tasks.deadline%TYPE default null,
	html_p		in pt_tasks.html_p%TYPE default 't',
	priority	in pt_tasks.priority%TYPE default null,
	workflow_key	in wf_workflows.workflow_key%TYPE default 'pt_task_wf',
	context_id	in acs_objects.context_id%TYPE,
	case_id		in wf_cases.case_id%TYPE default null,
	context_key	in varchar2 default null, 
	creation_date	in date default sysdate,
	creation_user	in acs_objects.creation_user%TYPE default null,
	creation_ip	in acs_objects.creation_ip%TYPE default null
    ) return pt_tasks.task_id%TYPE;

    -- Sometimes new workflow cases are created as a side effect of deletion
    procedure delete (
        task_id		in pt_tasks.task_id%TYPE,
	workflow_key	in wf_workflows.workflow_key%TYPE default 'pt_task_wf',
	context_id	in acs_objects.context_id%TYPE default null,
	case_id		in wf_cases.case_id%TYPE default null,
	context_key	in varchar2 default null, 
	creation_date	in date default sysdate,
	creation_user	in acs_objects.creation_user%TYPE default null,
	creation_ip	in acs_objects.creation_ip%TYPE default null
    );

    procedure move_branch (
        new_parent_id	in pt_tasks.task_id%TYPE,
	task_id		in pt_tasks.task_id%TYPE,
	workflow_key	in wf_workflows.workflow_key%TYPE default 'pt_task_wf',
	context_id	in acs_objects.context_id%TYPE,
	case_id		in wf_cases.case_id%TYPE default null,
	context_key	in varchar2 default null, 
	creation_date	in date default sysdate,
	creation_user	in acs_objects.creation_user%TYPE default null,
	creation_ip	in acs_objects.creation_ip%TYPE default null,
	create_new_cases in boolean default TRUE
    );

    procedure assign_task (
       task_id		in pt_tasks.task_id%TYPE,
       party_id		in acs_objects.object_id%TYPE
    );

    procedure unassign_task (
       task_id		in pt_tasks.task_id%TYPE,
       party_id		in acs_objects.object_id%TYPE
    );

    procedure start_workflow (
       task_id		in pt_tasks.task_id%TYPE,
       creation_user   in integer default null,
       creation_ip     in varchar2 default null,
       msg             in varchar2 default null
    );

    -- Returns 1 if the task is a parent task
    function is_parent (
        task_id	        in pt_tasks.task_id%TYPE,
	active_only	in char default 't'
    ) return number;

    function get_task_origin (
       task_id		in pt_tasks.task_id%TYPE
    ) return pt_tasks.task_id%TYPE;

    function is_ancestor (
       ancestor_id    in pt_tasks.task_id%TYPE,
       task_id	      in pt_tasks.task_id%TYPE
    ) return char;

    procedure drop_task (
        task_id		in pt_tasks.task_id%TYPE,
	workflow_key	in wf_workflows.workflow_key%TYPE default 'pt_task_wf',
	context_id	in acs_objects.context_id%TYPE,
	case_id		in wf_cases.case_id%TYPE default null,
	context_key	in varchar2 default null, 
	creation_date	in date default sysdate,
	creation_user	in acs_objects.creation_user%TYPE default null,
	creation_ip	in acs_objects.creation_ip%TYPE default null
    );

    function is_dropped (
        task_id		in pt_tasks.task_id%TYPE
    ) return char;

    -- Drop a task from a project (not delete)
    -- Takes package_id as an argument to hook into the package instance
    -- Also requires maybe_make child etc...

    procedure reactivate_task (
        task_id		in pt_tasks.task_id%TYPE,
	workflow_key	in wf_workflows.workflow_key%TYPE default 'pt_task_wf',
	context_id	in acs_objects.context_id%TYPE,
	case_id		in wf_cases.case_id%TYPE default null,
	context_key	in varchar2 default null, 
	creation_date	in date default sysdate,
	creation_user	in acs_objects.creation_user%TYPE default null,
	creation_ip	in acs_objects.creation_ip%TYPE default null
    );
    
end pt_task;
/
show errors

create or replace package body pt_task
as
    /* Private */
    
    -- If a task is a child, makes it a parent
    procedure child_to_parent (
         task_id in pt_tasks.task_id%TYPE 
    );

    procedure make_new_leaf (
        task_id		in pt_tasks.task_id%TYPE,
	workflow_key	in wf_workflows.workflow_key%TYPE default 'pt_task_wf',
	context_id	in acs_objects.context_id%TYPE,
	case_id		in wf_cases.case_id%TYPE default null,
	context_key	in varchar2 default null, 
	creation_date	in date default sysdate,
	creation_user	in acs_objects.creation_user%TYPE default null,
	creation_ip	in acs_objects.creation_ip%TYPE default null
    );
    
    procedure delete_recursive (
         task_id in pt_tasks.task_id%TYPE 
    );

    procedure delete_cases (
         task_id in pt_tasks.task_id%TYPE 
    );

    procedure delete_leaf_cases (
        parent_id	in pt_tasks.task_id%TYPE
    );

    procedure create_leaf_cases (
        parent_id	in pt_tasks.task_id%TYPE,
	workflow_key	in wf_workflows.workflow_key%TYPE default 'pt_task_wf',
	context_id	in acs_objects.context_id%TYPE,
	case_id		in wf_cases.case_id%TYPE default null,
	context_key	in varchar2 default null, 
	creation_date	in date default sysdate,
	creation_user	in acs_objects.creation_user%TYPE default null,
	creation_ip	in acs_objects.creation_ip%TYPE default null
    );

    -- Updates the state of a branch
    procedure update_branch_dropped_p (
        task_id		in pt_tasks.task_id%TYPE,
	dropped_p	in pt_tasks.dropped_p%TYPE
    );

    -- Update all parent states 
    procedure update_parent_dropped_p (
        task_id		in pt_tasks.task_id%TYPE,
	dropped_p	in pt_tasks.dropped_p%TYPE 
    );

    /* Public */

    function name (
        object_id	in acs_objects.object_id%TYPE
    ) return pt_tasks.name%TYPE
    is
         v_name pt_tasks.name%TYPE;
    begin
         select name into v_name
	 from pt_tasks
	 where task_id = name.object_id;
	
	 return v_name;
    end name; 

    function new (
        -- For pt_tasks
	task_id		in pt_tasks.task_id%TYPE default null,
	parent_id       in pt_tasks.task_id%TYPE,
	name		in pt_tasks.name%TYPE,
	description	in pt_tasks.description%TYPE,
	estimated_days	in pt_tasks.estimated_days%TYPE default 1,
	deadline	in pt_tasks.deadline%TYPE default null,
	html_p		in pt_tasks.html_p%TYPE default 't',
	priority	in pt_tasks.priority%TYPE default null,
	workflow_key	in wf_workflows.workflow_key%TYPE default 'pt_task_wf',
	context_id	in acs_objects.context_id%TYPE,
	case_id		in wf_cases.case_id%TYPE default null,
	context_key	in varchar2 default null, 
	creation_date	in date default sysdate,
	creation_user	in acs_objects.creation_user%TYPE default null,
	creation_ip	in acs_objects.creation_ip%TYPE default null
    ) return pt_tasks.task_id%TYPE
    is
	v_task_id		pt_tasks.task_id%TYPE;
	v_case_id		wf_cases.case_id%TYPE;
    begin
	-- When you create a task, it is always a leaf node
	-- Nullifying estimate and deadline and suspending case

        child_to_parent(pt_task.new.parent_id);

        v_task_id := acs_object.new (
	     object_id		=> pt_task.new.task_id,
	     object_type	=> 'pt_task',
	     creation_date	=> pt_task.new.creation_date,
	     creation_user	=> pt_task.new.creation_user,
	     creation_ip	=> pt_task.new.creation_ip, 
	     context_id		=> pt_task.new.context_id
	);

	insert into pt_tasks
            (task_id, name, description, 
	    parent_id, estimated_days, deadline, html_p,
	    priority)
        values 
	    (v_task_id, pt_task.new.name, pt_task.new.description, 
	     pt_task.new.parent_id, pt_task.new.estimated_days,
	    pt_task.new.deadline, pt_task.new.html_p,
	    priority );
	    
	-- We are going to wait for user feedback to start the case
	
        v_case_id := workflow_case.new (
            case_id         => pt_task.new.case_id,
            workflow_key    => pt_task.new.workflow_key,
            context_key     => pt_task.new.context_key,
            object_id       => v_task_id,
	    creation_date   => pt_task.new.creation_date,
	    creation_user   => pt_task.new.creation_user,
	    creation_ip     => pt_task.new.creation_ip
	);
   
	return v_task_id;
    end new;

    procedure delete (
        task_id	     in pt_tasks.task_id%TYPE,
	workflow_key	in wf_workflows.workflow_key%TYPE default 'pt_task_wf',
	context_id	in acs_objects.context_id%TYPE default null,
	case_id		in wf_cases.case_id%TYPE default null,
	context_key	in varchar2 default null, 
	creation_date	in date default sysdate,
	creation_user	in acs_objects.creation_user%TYPE default null,
	creation_ip	in acs_objects.creation_ip%TYPE default null
    )
    is
        v_parent_id pt_tasks.parent_id%TYPE;
	v_parent_p number;
    begin
       select parent_id into v_parent_id  from pt_tasks
       where task_id = pt_task.delete.task_id;

       -- Call the recursive tree delete function here
       delete_recursive (task_id => pt_task.delete.task_id);

       -- Finally fix the state of the workflow case
       v_parent_p := is_parent(v_parent_id);

       if v_parent_id is not null 
       then
            make_new_leaf (
	        task_id		=> v_parent_id,
		workflow_key	=> pt_task.delete.workflow_key,
		context_id	=> pt_task.delete.context_id,
		context_key	=> pt_task.delete.context_key,
		creation_date	=> pt_task.delete.creation_date,
		creation_user	=> pt_task.delete.creation_user,
		creation_ip	=> pt_task.delete.creation_ip
	    );
       end if;
    
    end delete;

    procedure move_branch (
        new_parent_id	in pt_tasks.task_id%TYPE,
	task_id		in pt_tasks.task_id%TYPE,
	workflow_key	in wf_workflows.workflow_key%TYPE default 'pt_task_wf',
	context_id	in acs_objects.context_id%TYPE default null,
	case_id		in wf_cases.case_id%TYPE default null,
	context_key	in varchar2 default null, 
	creation_date	in date default sysdate,
	creation_user	in acs_objects.creation_user%TYPE default null,
	creation_ip	in acs_objects.creation_ip%TYPE default null,
	create_new_cases in boolean default TRUE
    )
    is
    v_former_parent pt_tasks.task_id%TYPE;
    begin

        select parent_id into v_former_parent
	from pt_tasks
	where task_id = move_branch.task_id;
	-- Fix up the new parent
        child_to_parent (new_parent_id);

	update pt_tasks
	set parent_id	= move_branch.new_parent_id
	where task_id = move_branch.task_id;

	-- Fix up the old parent
	if create_new_cases = TRUE then
	    make_new_leaf (
            task_id		=> v_former_parent,
	    workflow_key	=> move_branch.workflow_key,
	    context_id		=> move_branch.context_id,
	    context_key		=> move_branch.context_key,
	    creation_date	=> move_branch.creation_date,
	    creation_user	=> move_branch.creation_user,
	    creation_ip		=> move_branch.creation_ip
	    );
	end if;
	
    end move_branch;

    procedure assign_task (
       task_id		in pt_tasks.task_id%TYPE,
       party_id		in acs_objects.object_id%TYPE
    )
    is
        cursor wf_task_cursor(v_case_id wf_cases.case_id%TYPE) 
	is
        select wt.task_id
        from wf_tasks wt
	where wt.case_id = v_case_id;

	wf_task_row wf_task_cursor%ROWTYPE;
	v_case_id wf_cases.case_id%TYPE;

	v_state wf_cases.state%TYPE;
    begin

       -- We do not need to check for duplicate assignments as it is
       -- enforced in the UI

       select case_id, state into v_case_id, v_state
       from wf_cases
       where object_id = assign_task.task_id;


       workflow_case.add_manual_assignment (
             case_id        => v_case_id,
	     transition_key => 'complete',
	     party_id	    => assign_task.party_id
       );

       -- Possible states are created, active, canceled, suspended,
       -- and finished, of these we need to update wf_task_assignments
       -- when the case is at active, canceled, suspended

       if v_state <> 'created' and v_state <> 'finished'
       then
           for wf_task_row in wf_task_cursor (v_case_id)
	   loop
               workflow_case.add_task_assignment (
	           task_id       => wf_task_row.task_id,
		   party_id      => assign_task.party_id
	       );
           end loop;
       end if;

    end assign_task; 

    procedure unassign_task (
       task_id		in pt_tasks.task_id%TYPE,
       party_id		in acs_objects.object_id%TYPE
    )
    is
        cursor wf_task_cursor(v_case_id wf_cases.case_id%TYPE) 
	is
        select wt.task_id
        from wf_tasks wt
	where wt.case_id = v_case_id;

	wf_task_row wf_task_cursor%ROWTYPE;
	
	v_case_id wf_cases.case_id%TYPE;

	v_state wf_cases.state%TYPE;
    begin

       -- We do not need to check for duplicate assignments as it is
       -- enforced in the UI

       select case_id, state into v_case_id, v_state
       from wf_cases
       where object_id = unassign_task.task_id;


       workflow_case.remove_manual_assignment (
             case_id        => v_case_id,
	     transition_key => 'complete',
	     party_id	    => unassign_task.party_id
       );

       -- Possible states are created, active, canceled, suspended,
       -- and finished, of these we need to update wf_task_assignments
       -- when the case is at active, canceled, suspended
       
       if v_state <> 'created' and v_state <> 'finished'
       then
           for wf_task_row in wf_task_cursor (v_case_id)
	   loop
               workflow_case.remove_task_assignment (
	           task_id       => wf_task_row.task_id,
		   party_id      => unassign_task.party_id
	       );

	       
	       -- If the assignee is also the holding user we need to
	       -- revert the state to go back to needs to be started

           end loop;
       end if;

    end unassign_task;

    function is_parent (
        task_id	       in pt_tasks.task_id%TYPE,
	active_only	in char default 't'
    ) return number
    is 
        v_boolean number;
    begin
        if active_only = 't'
	then
	     select decode (count(*), 0, 0, 1) into v_boolean
	     from pt_tasks
	     where parent_id = is_parent.task_id
	     and dropped_p = 'f';

	else
	     select decode (count(*), 0, 0, 1) into v_boolean
	     from pt_tasks
	     where parent_id = is_parent.task_id;
	end if;

	return v_boolean;

    end is_parent;

    function get_task_origin (
       task_id		in pt_tasks.task_id%TYPE
    ) return pt_tasks.task_id%TYPE
    is
         v_root_id pt_tasks.task_id%TYPE;
    begin
         select task_id into v_root_id
	 from pt_tasks
	 where parent_id is null
	 start with task_id = get_task_origin.task_id
	 connect by task_id = prior parent_id;
	 
	 return v_root_id;
    end get_task_origin;

    procedure start_workflow (
       task_id		in pt_tasks.task_id%TYPE,
       creation_user   in integer default null,
       creation_ip     in varchar2 default null,
       msg             in varchar2 default null
    ) 
    is
	cursor case_cursor(v_task_id pt_tasks.task_id%TYPE) is
	select case_id from wf_cases
	where object_id = v_task_id 
	and workflow_key = 'pt_task_wf';
    
	case_row case_cursor%ROWTYPE;
    begin
       
       for case_row in case_cursor(start_workflow.task_id)
       loop
            workflow_case.start_case (
	         case_id        => case_row.case_id,
		 creation_user  => start_workflow.creation_user,
		 creation_ip	=> start_workflow.creation_ip,
		 msg		=> start_workflow.msg
	    );
       end loop;

    end start_workflow;

    function is_ancestor (
       ancestor_id    in pt_tasks.task_id%TYPE,
       task_id	      in pt_tasks.task_id%TYPE
    ) return char
    is
	v_boolean char;
    begin
       -- Upward traversal

       select decode (count(*), 0, 'f', 't') into v_boolean
       from pt_tasks
       where task_id = is_ancestor.ancestor_id
       start with task_id = is_ancestor.task_id
       connect by task_id = prior parent_id;
       
       return v_boolean;
    end is_ancestor;

    procedure drop_task (
        task_id		in pt_tasks.task_id%TYPE,
	workflow_key	in wf_workflows.workflow_key%TYPE default 'pt_task_wf',
	context_id	in acs_objects.context_id%TYPE default null,
	case_id		in wf_cases.case_id%TYPE default null,
	context_key	in varchar2 default null, 
	creation_date	in date default sysdate,
	creation_user	in acs_objects.creation_user%TYPE default null,
	creation_ip	in acs_objects.creation_ip%TYPE default null
    )
    is
	v_parent pt_tasks.task_id%TYPE;
    begin
        -- The only thing we do here is update status
	select parent_id into v_parent
	from pt_tasks
	where task_id = drop_task.task_id;

	update_branch_dropped_p (
	    task_id	=> drop_task.task_id,
	    dropped_p   => 't'
	);

	make_new_leaf (
            task_id		=> v_parent,
	    workflow_key	=> drop_task.workflow_key,
	    context_id		=> drop_task.context_id,
	    context_key		=> drop_task.context_key,
	    creation_date	=> drop_task.creation_date,
	    creation_user	=> drop_task.creation_user,
	    creation_ip		=> drop_task.creation_ip
	    );

	-- Maybe make child

	-- But since all these tasks are dropped none of it should have cases
	-- we delete the cases for the leaf tasks

	delete_leaf_cases (
	    parent_id     => drop_task.task_id
	);

    end drop_task;

    function is_dropped (
        task_id		in pt_tasks.task_id%TYPE
    ) return char
    is
	v_is_dropped char;
    begin		
        select dropped_p into  v_is_dropped
	from pt_tasks
	where task_id = is_dropped.task_id;
	
	return v_is_dropped;
    end is_dropped;

    procedure reactivate_task (
        task_id		 in pt_tasks.task_id%TYPE,
	workflow_key	in wf_workflows.workflow_key%TYPE default 'pt_task_wf',
	context_id	in acs_objects.context_id%TYPE default null,
	case_id		in wf_cases.case_id%TYPE default null,
	context_key	in varchar2 default null, 
	creation_date	in date default sysdate,
	creation_user	in acs_objects.creation_user%TYPE default null,
	creation_ip	in acs_objects.creation_ip%TYPE default null
    )
    is
	v_parent_id pt_tasks.task_id%TYPE;
    begin
        -- We do not create new cases in the dropped branch
	-- Nullify fields in the parent task
	
	select parent_id into v_parent_id
	from pt_tasks
	where task_id = reactivate_task.task_id;

	child_to_parent(v_parent_id);

	update_branch_dropped_p (
	    task_id => reactivate_task.task_id,
	    dropped_p => 'f'
	);
	
	-- We also need to update the state of parents

	update_parent_dropped_p (
	    task_id => reactivate_task.task_id,
	    dropped_p => 'f'
	);

	-- We recreate cases for leaves that were deleted 
	create_leaf_cases (
	    parent_id	  => reactivate_task.task_id,
	    workflow_key  => reactivate_task.workflow_key,	
	    context_id    => reactivate_task.context_id,
	    case_id	  => reactivate_task.case_id,	
	    context_key   => reactivate_task.context_key,	
	    creation_date => reactivate_task.creation_date,	
	    creation_user => reactivate_task.creation_user,	
	    creation_ip   => reactivate_task.creation_ip
	);
    end reactivate_task;

    /* Private */

    procedure child_to_parent (
         task_id in pt_tasks.task_id%TYPE 
    ) 
    is
        v_parent_p number;
    begin
       v_parent_p := is_parent(
                           task_id => task_id,	
			   active_only => 't'
		      );

       if v_parent_p <> 1 
       then

           -- The task is a child and must be nullified
	   update pt_tasks
	       set estimated_days = null,
	       deadline = null,
	       leaf_p = 'f',
	       actual_days = null
	   where task_id = pt_task.child_to_parent.task_id;

	   delete_cases(pt_task.child_to_parent.task_id);
       
           -- We also delete the assignments
       end if;
    end child_to_parent;

    procedure make_new_leaf (
        task_id		in pt_tasks.task_id%TYPE,
	workflow_key	in wf_workflows.workflow_key%TYPE default 'pt_task_wf',
	context_id	in acs_objects.context_id%TYPE,
	case_id		in wf_cases.case_id%TYPE default null,
	context_key	in varchar2 default null, 
	creation_date	in date default sysdate,
	creation_user	in acs_objects.creation_user%TYPE default null,
	creation_ip	in acs_objects.creation_ip%TYPE default null
    ) 
    is
        v_case_count		number;
	v_case_id		wf_cases.case_id%TYPE;
	v_parent_p		number;
    begin
       -- If a task is a child task, does the appropriate things to 
       -- make it a leaf task

       -- A task is a leaf task if it doesn't have any children that
       -- are not 'dropped'
       v_parent_p := is_parent(
                           task_id => task_id,	
			   active_only => 't'
		      );

       if v_parent_p <>  1
       then
            -- Task is a newly made leaf task
            -- If the new leaf task doesn't have a wf_case, create one
	    select count(*) into v_case_count
	    from wf_cases
	    where object_id = make_new_leaf.task_id;

	    update pt_tasks
	    set leaf_p ='t'
	    where task_id = make_new_leaf.task_id;

	    if v_case_count = 0 then

	    v_case_id := workflow_case.new (
                 case_id         => pt_task.make_new_leaf.case_id,
		 workflow_key    => pt_task.make_new_leaf.workflow_key,
		 context_key     => pt_task.make_new_leaf.context_key,
		 object_id       => pt_task.make_new_leaf.task_id,
		 creation_date   => pt_task.make_new_leaf.creation_date,
		 creation_user   => pt_task.make_new_leaf.creation_user,
		 creation_ip     => pt_task.make_new_leaf.creation_ip
	      );
   
	    end if;
       end if;
    end make_new_leaf;

    procedure delete_recursive (
        task_id in pt_tasks.task_id%TYPE 
    )
    is
        cursor child_cursor(v_task_id pt_tasks.task_id%TYPE) is 
	select task_id from pt_tasks where parent_id = v_task_id;
    
	child_row child_cursor%ROWTYPE;
    begin
        for child_row in child_cursor(delete_recursive.task_id)
	loop
            delete_cases( task_id => child_row.task_id);
	    delete_recursive(task_id => child_row.task_id);
	end loop;

        delete
        from pt_tasks 
        where task_id = pt_task.delete_recursive.task_id;

        acs_object.delete (
           object_id	 => pt_task.delete_recursive.task_id
        );

    end delete_recursive;

    procedure delete_cases (
         task_id in pt_tasks.task_id%TYPE 
    )
    is
	cursor case_cursor(v_task_id pt_tasks.task_id%TYPE)
	is
	select case_id from wf_cases where object_id = v_task_id
	and workflow_key = 'pt_task_wf';
	
	case_row case_cursor%ROWTYPE;
    begin
       for case_row in case_cursor(delete_cases.task_id)
       loop
           workflow_case.delete ( case_id => case_row.case_id );
       end loop;

    end delete_cases;

    -- For dropping and reactivating tasks
    procedure update_branch_dropped_p (
        task_id		in pt_tasks.task_id%TYPE,
	dropped_p	in pt_tasks.dropped_p%TYPE
    )
    is
        cursor
	branch_cursor(v_task_id pt_tasks.task_id%TYPE) is
	select task_id
	from pt_tasks
	start with task_id = v_task_id
	connect by parent_id = prior task_id;
    
	branch_row branch_cursor%ROWTYPE;
    begin
        for branch_row in branch_cursor(task_id)
	loop

	    update pt_tasks
	    set dropped_p = update_branch_dropped_p.dropped_p
	    where task_id = branch_row.task_id;

	end loop;
    end update_branch_dropped_p;

    procedure update_parent_dropped_p (
        task_id		in pt_tasks.task_id%TYPE,
	dropped_p	in pt_tasks.dropped_p%TYPE 
    )
    is	
	cursor parent_cursor(v_task_id pt_tasks.task_id%TYPE)
	is 
	select parent_id
	from pt_tasks
	start with task_id = v_task_id
	connect by task_id = prior parent_id;
    
	parent_row parent_cursor%ROWTYPE;
    begin

        for parent_row in parent_cursor(update_parent_dropped_p.task_id)
	loop
	    update pt_tasks
	    set dropped_p = update_parent_dropped_p.dropped_p
	    where task_id = parent_row.parent_id;
	end loop;

    end update_parent_dropped_p;

    procedure delete_leaf_cases (
        parent_id	in pt_tasks.task_id%TYPE
    )
    is
	cursor leaf_cursor (v_parent_id pt_tasks.task_id%TYPE) is
	select task_id 
	from pt_tasks
	where pt_task.is_parent(task_id) = 0
	start with task_id = v_parent_id
	connect by parent_id = prior task_id;

	leaf_row leaf_cursor%ROWTYPE;
    
	v_case_id		wf_cases.case_id%TYPE;
    begin
	-- This step is not completely necessary since only tasks in
	-- the active cases can be started. However, it is the right thing to
	-- do.
        for leaf_row in leaf_cursor(delete_leaf_cases.parent_id)
	loop	     
	   update pt_tasks
	   set leaf_p = 'f'
	   where task_id = leaf_row.task_id;

	   pt_task.delete_cases (leaf_row.task_id);
	end loop;

    end delete_leaf_cases;

    procedure create_leaf_cases (
        parent_id	in pt_tasks.task_id%TYPE,
	workflow_key	in wf_workflows.workflow_key%TYPE default 'pt_task_wf',
	context_id	in acs_objects.context_id%TYPE,
	case_id		in wf_cases.case_id%TYPE default null,
	context_key	in varchar2 default null, 
	creation_date	in date default sysdate,
	creation_user	in acs_objects.creation_user%TYPE default null,
	creation_ip	in acs_objects.creation_ip%TYPE default null
    )
    is
        cursor leaf_cursor (v_parent_id in pt_tasks.task_id%TYPE) is
	select task_id 
	from pt_tasks
	where pt_task.is_parent(task_id) = 0
	start with task_id = v_parent_id
	connect by parent_id = prior task_id;

	leaf_row    leaf_cursor%ROWTYPE;
    
	v_case_id	wf_cases.case_id%TYPE;
    begin
	-- This step is not completely necessary since only tasks in
	-- the active cases can be started. However, it is the right thing to
	-- do.
        for leaf_row in leaf_cursor(create_leaf_cases.parent_id)
	loop	
	    update pt_tasks
	    set leaf_p = 't'
	    where task_id = leaf_row.task_id;

            v_case_id := workflow_case.new (
            case_id         => create_leaf_cases.case_id,
            workflow_key    => create_leaf_cases.workflow_key,
            context_key     => create_leaf_cases.context_key,
            object_id       => leaf_row.task_id,
	    creation_date   => create_leaf_cases.creation_date,
	    creation_user   => create_leaf_cases.creation_user,
	    creation_ip     => create_leaf_cases.creation_ip
     	   );
	end loop;

    end create_leaf_cases;

end pt_task;
/
show errors


















