/* 

   Basic creation and deletion test for pt_task. Tests for
   sql/pt-task-package.sql

   Requires utPL/SQL

   @author Nobuko Asakai (nasakai@arsdigita.com)
   @creation-date 01-19-2001
   @cvs-id $Id: pt-task-test.sql,v 1.1.1.1 2001/04/10 21:00:12 nasakai Exp $
   
*/

set serveroutput on size 1000000 format wrapped;

create or replace package ut_pt_task
is
    procedure ut_setup;
        
    procedure ut_teardown;

    procedure new;

    procedure task_delete;

    procedure create_tree (
        parent_id   in pt_tasks.parent_id%TYPE default null
    );
    -- Container procedure
    procedure ut_pt_task;

    procedure move_branch;

    procedure delete_branch;

    procedure assign_task;

    procedure unassign_task;

    procedure drop_task;

    procedure reactivate_task;

    procedure is_parent;
    
end ut_pt_task;
/
show errors

create or replace package body ut_pt_task
is 
    v_task_id pt_tasks.task_id%TYPE;
    v_parent pt_tasks.task_id%TYPE;
    v_child_one  pt_tasks.task_id%TYPE;
    v_child_two pt_tasks.task_id%TYPE;
    v_grandchild pt_tasks.task_id%TYPE;
    v_grandchild_two pt_tasks.task_id%TYPE;
    v_number number;
    v_count number;
   
    -- This knows if you are leaf based on estimate
    procedure    leaf_check (
        msg_in     in varchar2,
	task_id	   in pt_tasks.task_id%TYPE,
	leaf_p	   in boolean default true
    );

    procedure ut_setup
    is
    begin
        ut_teardown;
	dbms_output.put_line('Cleaning old data and setting up...');
    end ut_setup;

    procedure ut_teardown
    is
	cursor root_cursor
	is 
	select task_id from pt_tasks
	where parent_id is null;

	root_row root_cursor%ROWTYPE;
    begin
        dbms_output.put_line ('Tearing down old data...');
	
	for root_row in root_cursor 
	loop
	    pt_task.delete(
	        task_id => root_row.task_id,
		context_id => -1);
	end loop;
	
	select count(*) into v_number
	from pt_tasks;

	utAssert.eq (
	    msg_in => 'Done tearing down',
	    check_this_in => v_number,
	    against_this_in => 0
        );
    end ut_teardown;

    procedure new 
    is
    v_count number;
    v_varchar varchar2(400);
    begin
        dbms_output.put_line ('Checking for basic creation');

	v_task_id := pt_task.new (
	    parent_id	       => null,
	    task_id	       => -1000,
	    name	       => 'obj_id_test',
	    description	       => 'test',	
	    context_id	       => -1
	    );

	utAssert.eq (
	    msg_in		=> 'creating a pt_task object',
	    check_this_in	=>  v_task_id,
	    against_this_in	=> -1000
	);
	
	-- name function test
	utAssert.eq (
	    msg_in		=> 'testing name function',
	    check_this_in	=> pt_task.name ( object_id => -1000 ),
	    against_this_in	=> 'obj_id_test'
	);

	-- now testing that it inserts in all the proper places
	select decode (count(*), 0, 0, 1) into v_count
	from pt_tasks where task_id = -1000;

	utAssert.eq (
	    msg_in		=> 'Checking input into pt_tasks',
	    check_this_in	=> v_count,
	    against_this_in	=> 1
	);

	-- Checking against acs_workflow, the status might change depending...
	-- this means the call to workflow_case.new is sucessful
	-- could be 'created' 'active' 'suspended' 'canceled' 'finished'


	select count(*) into v_count
	from pt_tasks
	where parent_id = v_task_id;

	utAssert.eq (
	    msg_in		=> 'There should not be any tree',
	    check_this_in	=> v_count,
	    against_this_in	=> 0
	);
    end new;

    procedure task_delete
    is
    begin
	-- Deletion test

	dbms_output.put_line ('checking deletion');
	pt_task.delete (
	    task_id		=> -1000,
	    context_id		=> -1
	);
	
	-- is it really gone?
	select decode (count(*), 0, 0, 1) into v_count
	from pt_tasks where task_id = -1000;

	utAssert.eq (
	    msg_in		=> 'Checking input into pt_tasks',
	    check_this_in	=> v_count,
	    against_this_in	=> 0
	);

    end task_delete;

    procedure create_tree (
        parent_id   in pt_tasks.parent_id%TYPE default null
    )
    is
    begin
	 /* Creating standard task tree 
	    P
	    -c
	    --g
	    --g2
	    -c */
	 if create_tree.parent_id is null then
	     ut_teardown;
	     v_parent :=  pt_task.new (
	         parent_id => null,
		 name => 'parent',
		 description => 'Parent task',
		 context_id => -1
             );
	 else
	      v_parent := create_tree.parent_id;
	 end if;
	 
	 v_child_one :=  pt_task.new (
	     name => 'child_one',
	     description => 'Child one',
	     parent_id => v_parent,
	     context_id => -1
	 );

	 -- Make sure that the root is not a child anymore
	 leaf_check (
	     msg_in	=> 'The root should not be a child anymore',
	     task_id	=> v_parent,
	     leaf_p	=> FALSE
	 );
	 -- Second Child
	 v_child_two :=  pt_task.new (
	     name => 'child_two',
	     description => 'Child Two',
	     parent_id => v_parent,
	     context_id => -1
	 );

	 -- Child one's child

	 v_grandchild :=  pt_task.new (
	     name => 'grandchild',
	     description => 'child of child one',
	     parent_id => v_child_one,
	     context_id => -1
	 );

	 -- Child one's second child

	 v_grandchild_two :=  pt_task.new (
	     name => 'grandchild_two',
	     description => 'of child child one',
	     parent_id => v_child_one,
	     context_id => -1
	 );

	 select level into v_number
	 from pt_tasks
	 where task_id = v_grandchild_two
	 start with parent_id is null
	 connect by parent_id = prior task_id;
	 
	 utAssert.eq (
	 msg_in	    => 'The level for grandchild two should be 3',
	 check_this_in => v_number,
	 against_this_in => 3
	 );
    end;

    procedure delete_branch
    is
    begin
    -- Create tree and delete different parts of it
        create_tree;

	 /* Creating standard task tree 
	    P
	    -c
	    --g
	    --g2
	    -c */

	pt_task.delete ( 
	    task_id => v_grandchild_two,
	    context_id => -1);
	

	select count(*) into v_number from pt_tasks;	
	
	utAssert.eq (
	    msg_in          => 'Deleted g2',
	    check_this_in   =>  v_number,
	    against_this_in  =>  4);
	
	pt_task.delete (
	task_id => v_child_one,
        context_id => -1);

	select count(*) into v_number from pt_tasks;	
	
	utAssert.eq (
	    msg_in          => 'Deleted c1',
	    check_this_in   =>  v_number,
	    against_this_in  =>  2);


	pt_task.delete (
	task_id => v_child_two,
	context_id => -1
	);

	leaf_check (
	    msg_in => 'C1 and C2 deleted',
	    task_id => v_parent,
	    leaf_p => TRUE
	);
	
	pt_task.delete (
	task_id => v_parent,
	context_id => -1 );

	select count(*) into v_number from pt_tasks;	
	
	utAssert.eq (
	    msg_in          => 'Deleted root node',
	    check_this_in   =>  v_number,
	    against_this_in  =>  0);

	dbms_output.put_line ('Deleted the entire tree');
    end;
   
    procedure move_branch
    is
    begin
	 /* Creating standard task tree 
	    P
	    -c
	    --g
	    --g2
	    -c */

	create_tree;



	select level into v_number
	from pt_tasks
	where task_id = v_grandchild_two
	start with task_id = v_parent
	connect by parent_id = prior task_id;

	utAssert.eq (
	    msg_in          => 'Initial',
	    check_this_in   =>  v_number,
	    against_this_in  =>  3);

	pt_task.move_branch (
	    new_parent_id => v_child_two,
	    task_id => v_child_one,
	    context_id => -1 );

	    	    
	 /* Moved task_tree
	    P
	    -c 2
 	    ---c 1
	    -----g
	    -----g2*/

	select level into v_number
	from pt_tasks
	where task_id = v_grandchild_two
	start with task_id = v_parent
	connect by parent_id = prior task_id;

	select count(*) into v_count
	from wf_cases
	where object_id = v_child_one;
	
	dbms_output.put_line ('child_one''s case count is '|| v_count);

	utAssert.eq (
	    msg_in          => 'Moved c1 branch under c2, level check',
	    check_this_in   =>  v_number,
	    against_this_in  =>  4);

	dbms_output.put_line ('child_one''s case count is '|| v_count);
	leaf_check (
	    msg_in => 'Move c1 under c2',
	    task_id => v_child_two,
	    leaf_p => FALSE
	);

	dbms_output.put_line ('child_one''s case count is '|| v_count);
	leaf_check (
	    msg_in => 'Move c1 under c2 child one check ',
	    task_id => v_child_one,
	    leaf_p => FALSE
	);

	 /* Moved task_tree
	    P
	    -c 2
	    ---g
	    ---g2
 	    ---c 1
	    */

	 
	pt_task.move_branch (
	    new_parent_id => v_child_two,
	    task_id => v_grandchild,
	    context_id => -1  );

	pt_task.move_branch (
	    new_parent_id => v_child_two,
	    task_id => v_grandchild_two,
	    context_id => -1  );

	leaf_check (
	    msg_in => 'Move g1 and g2, checking child one ',
	    task_id => v_child_one,
	    leaf_p => TRUE
	);

	select level into v_number
	from pt_tasks
	where task_id = v_grandchild_two
	start with task_id = v_parent
	connect by parent_id = prior task_id;

	utAssert.eq (
	    msg_in          => 'Moved c1 branch under P, level check',
	    check_this_in   =>  v_number,
	    against_this_in  =>  3);
	    
	-- Iterations
	pt_task.move_branch (
	    new_parent_id => v_child_one,
	    task_id => v_grandchild_two,
	    context_id => -1  );

	leaf_check (
	    msg_in => 'Move g2 back under c1',
	    task_id => v_child_one,
	    leaf_p => FALSE
	);

	dbms_output.put_line ('...done testing move');

    end;

    /* Private */
    -- procedure for checking the place the workflow is at
    -- we will use tokens for that

    procedure leaf_check (
        msg_in     in varchar2,
	task_id	   in pt_tasks.task_id%TYPE,
	leaf_p	   in boolean default TRUE
    ) is
        v_count number;
	is_leaf char;
    begin
    
        select decode(count(*), 0, NULL, 1) into v_count
        from wf_cases
        where object_id = task_id;

	select leaf_p into is_leaf
	from pt_tasks
	where task_id = leaf_check.task_id;

	if leaf_p then

	    utAssert.isnotNull (
	        msg_in => msg_in || 'Leaves must have cases'  ,	
		check_this_in => v_count
		);

	    utAssert.eq (
	        check_this_in =>is_leaf,
		against_this_in => 't',
		msg_in =>msg_in || ' Leaf p is t' 
	    );
	else 

	    utAssert.isNull (
	        msg_in => msg_in || 'Parent does not have cases',	
		 check_this_in => v_count
	    );

	    utAssert.eq (
	        check_this_in =>is_leaf,
		against_this_in => 'f',
		Msg_in =>msg_in || ' Leaf p is f'  
	    );
	end if;
    end;

    procedure assign_task 
    is
    v_party_id number;
    begin
        create_tree;
	
	-- -1 is the default everyone
	pt_task.assign_task (
	    task_id  =>  v_grandchild,
	    party_id => -1
	);

	-- Check for assignment creation
	
	select party_id into v_party_id
	from wf_case_assignments wca, wf_cases wc
	where wc.object_id = v_grandchild
	and wc.case_id = wca.case_id;

	utAssert.eq (
	    msg_in	    => 'Checking for initial assignment creation',	
	    check_this_in   => v_party_id,
	    against_this_in => -1
	);

	pt_task.start_workflow (
	    task_id => v_grandchild
	);

	-- At this point the wf_task should also have the assignment

	select count(wta.party_id) into v_count
	from wf_task_assignments wta, wf_cases wc, wf_tasks wt
	where wt.case_id = wc.case_id
	and wt.task_id = wta.task_id
	and wc.object_id = v_grandchild;

	utAssert.eq (
	    msg_in	    => 'Checking for assignment in wf_task_assignments',	
	    check_this_in   => v_count,
	    against_this_in => 1
	);


	dbms_output.put_line ('Adding assignment to an active case');
	pt_task.assign_task (
	    task_id  => v_grandchild,
	    party_id => -2
	);

	select count(wca.party_id) into v_count
	from wf_case_assignments wca, wf_cases wc
	where wc.object_id = v_grandchild
	and wc.case_id = wca.case_id;

	utAssert.eq (
	    msg_in	    => 'Checking for wf_case assignment after case is started',	
	    check_this_in   => v_count,
	    against_this_in => 2
	);

	select count(wta.party_id) into v_count
	from wf_task_assignments wta, wf_cases wc, wf_tasks wt
	where wt.case_id = wc.case_id
	and wt.task_id = wta.task_id
	and wc.object_id = v_grandchild;

	utAssert.eq (
	    msg_in	    => 'Checking for wf_task assignment after case is started',	
	    check_this_in   => v_count,
	    against_this_in => 2
	);

	-- we also need to check for wf_task_assignments

	dbms_output.put_line ('done');
	
	-- Now check for update in wf
    end;

    procedure unassign_task 
    is
    v_party_id number;
    begin
	assign_task;
	
	pt_task.unassign_task (
	    task_id => v_grandchild,
	    party_id => -2
	);

	select count(*) into v_party_id
	from wf_case_assignments wca, wf_cases wc
	where wc.object_id = v_grandchild
	and wc.case_id = wca.case_id
	and party_id = -2;
	
	utAssert.eq (
	    msg_in	    => 'Checking for update in workflow for unassigntment',	
	    check_this_in   => v_party_id,
	    against_this_in => 0
	);

	dbms_output.put_line ('--> Done assigning');
    end;

    procedure drop_task
    is
    v_dropped_p pt_tasks.dropped_p%TYPE;
    begin

       create_tree;

       pt_task.drop_task (
          task_id    => v_grandchild_two,
	  context_id => -1
       );

       select count(*) into v_count
       from pt_tasks
       where task_id = v_grandchild_two
       and dropped_p = 'f'
       start with task_id = v_parent
       connect by parent_id = prior task_id;


       utAssert.eq (
           msg_in => 'Dropped grandchild two',
	   check_this_in => v_count,
	   against_this_in => 0
       );
       
       -- G2 should not have a case now

       select count(*) into v_count
       from wf_cases
       where object_id = v_grandchild_two;
       
       utAssert.eq (
           msg_in => 'No case for g2 after droppage',
	   check_this_in => v_count,
	   against_this_in => 0
       );
       
       pt_task.drop_task (
          task_id    => v_grandchild,
	  context_id => -1
       );

       leaf_check (
          task_id => v_child_one,
	  msg_in  => 'Dropped g1 and g2',
	  leaf_p  => TRUE
       );

       create_tree;

       pt_task.drop_task (
          task_id => v_child_one,
	  context_id => -1
       );

       select dropped_p into v_dropped_p
       from pt_tasks
       where task_id = v_grandchild;
       
       utAssert.eq (
           msg_in	   => 'C1 dropped, and g1 dropped_p should be dropped',
	   check_this_in   => v_dropped_p,
	   against_this_in => 't'
       );
	dbms_output.put_line ('--> Done Dropping');
    end;

    procedure reactivate_task
    is
       v_dropped_p char;
    begin
       ut_teardown;
        
       create_tree;

        /* Creating standard task tree 
	    P
	    -c
	    --g
	    --g2
	    -c */

       pt_task.drop_task (
           task_id => v_child_one,
	   context_id => -1
       );       

       pt_task.reactivate_task (
           task_id => v_grandchild,
	   context_id => -1
       );
       
       leaf_check (
          task_id => v_grandchild,
	  msg_in  => 'Reactivated g1  after dropping c1',
	  leaf_p  => TRUE
       );

       select dropped_p into v_dropped_p 
       from pt_Tasks
       where task_id = v_child_one;

       utAssert.eq (
          msg_in => 'c1 get''s reactivated too',
	  check_this_in => 'f',
	  against_this_in => v_dropped_p
       );
	dbms_output.put_line ('--> Done Reactivating');
    end;

    procedure is_parent 
    is
    begin
       create_tree;
       
       utAssert.eq (
          msg_in => 'Active parent',
	  check_this_in => pt_task.is_parent (v_parent),
	  against_this_in => 1
       );

       utAssert.eq (
          msg_in => 'Active parent',
	  check_this_in => pt_task.is_parent (v_parent, 'f'),
	  against_this_in => 1
       );
    end;

    procedure set_priority
    is
    v_priority number;
    v_task_two pt_tasks.task_id%TYPE;
    begin
        -- First test as part of pt_task.new function
	-- Feature added on April 09 2001 nasakai@arsdigita.com

	ut_teardown;
	-- Without priority column, this is tested with the other new unit test
	v_task_id := pt_task.new (
	    parent_id	       => null,
	    task_id	       => -1000,
	    name	       => 'priority_test',
	    description	       => 'Test for insert without Priority',	
	    context_id	       => -1
	    );

	-- Update to an existing task
	update pt_tasks set priority = 1;

	select priority into v_priority from pt_tasks 
	where task_id = v_task_id;

	utAssert.eq (
	     msg_in		=> 'Checking update value',
	     check_this_in 	=> v_priority,
	     against_this_in	=> 1
	);

	-- With priority column, but without entry in the priorities table
	v_task_two := pt_task.new (
	    parent_id	       => null,
	    task_id	       => -1000,
	    priority	       => 2,
	    name	       => 'priority_test',
	    description	       => 'Test for insert without Priority',	
	    context_id	       => -1
	);
	
	select priority into v_priority from pt_tasks
	where task_id = v_task_two;

	utAssert.eq (
	     msg_in		=> 'Checking update value',
	     check_this_in 	=> v_priority,
	     against_this_in	=> 2
	);

    end;

    /* Container*/
    procedure ut_pt_task
    is
    v_count number;
    v_varchar varchar2(400);
    begin

 	dbms_output.put_line ('--> Basic creation');
         new;
	
 	task_delete;
	
 	dbms_output.put_line ('--> Tree creation');
 	create_tree;

 	dbms_output.put_line ('--> Deleting branches');	
 	delete_branch;

 	dbms_output.put_line ('--> Moving branches');	
 	move_branch;

 	dbms_output.put_line ('--> Checking assignments');	
 	assign_task;

-- 	unassign_task;
 	dbms_output.put_line ('--> Checking drop');	
 	drop_task;
	
 	dbms_output.put_line ('--> Checking reactivate');	
 	reactivate_task;

 	dbms_output.put_line ('--> Checking for is_parent');	
	is_parent;
    end ut_pt_task;
	
end ut_pt_task;
/
show errors

exec utplsql.test('pt_task', recompile_in => false);

-- drop package ut_pt_task;







