ad_library {
    Tcl library for project-tracker
}

ad_page_contract_filter pt_task_id {name value} {
    Checks whether the value (assumed to be a integer) is a valid id of
    a task that belongs to the project-tracker instance.
} {
    set package_id [ad_conn package_id]

    set id_valid_p [db_exec_plsql check_task_id {
	begin

        :1 := pt_project.task_valid_p (
	      package_id => :package_id, 
	      task_id => :value
	); 
	end;
    }]

	ns_log notice "\nid_valid_p $id_valid_p"

    if { [string equal $id_valid_p 0] } {
        ad_complain "$name: $value is not a valid task id for 
	this project-tracker instance ($package_id)"
        return 0
    }
    return 1
}

ad_page_contract_filter pt_estimated_days {name value} {
    Checks for the validity of estimated_days. We allow up to 2 decimal points
} {
    if {![regexp {^\+?([0-9]+(\.[0-9][0-9]?)?|\.[0-9][0-9]?)$} $value]} {
	ad_complain "$name: $value is not a valid number"
        return 0
    } else {
        return 1
    }
}

ad_page_contract_filter pt_priority_id {name value} {
    Checks for the validity of estimated_days. We allow up to 2 decimal points
} {
    set package_id [ad_conn package_id]
    
    db_1row package_priority_id_check {
	select count(*) as valid_p
	from pt_priorities
	where package_id = :package_id
	and priority_id = :value
	
    }
	
    if {!$valid_p} {
	ad_complain "$value not a valid priority for
	package instance $package_id"
	return 0
    }
    
    return 1
}

ad_page_contract_filter max_string_length {name value max_length} {
    Checks for the maximum length string
} {
    set string_length [string length $value] 
    if {[expr {$string_length > $max_length}] } {
	ad_complain "$name: $value is too long (must be shorter than $max_length)"
        return 0
    } else {
	return 1
    }
}

ad_proc -private pt_get_root {
    {package_id ""}
} {
    Returns the root_id for the PT instance, null if it does not exist
} {
    if {![exists_and_not_null package_id]} {
	set package_id [ad_conn package_id]
    }
    
    db_1row get_root "
    select pt_project.get_root(:package_id) as root_id from dual"

    return $root_id
}

ad_proc -private append_anchor {
    {-anchor ""}
    {query_string ""}
} { 
    A wrapper for adding the anchor tags in the urls. Urlencodes the
    pound mark. The url parameter must be the full url
} {
    
    # We say &pt#anchor because IE 5 will read the anchor as part of the
    # last query string. We can't say &#anchor bacause &# translates
    # to a question mark. pt is is a dummy value.
    
    if {[exists_and_not_null anchor]} {
	    append query_string "&pt#${anchor}"
    }

    return $query_string
}

ad_proc -deprecated indent_task { depth } {
    set return_string ""

    for { set i 1 } { $i < $depth } { incr i } {
	append return_string "&nbsp;&nbsp;&nbsp;&nbsp;"
    }

    return $return_string
}


ad_proc -private drop_option_helper {
    {-root_id} 
    {-link_type}
    {-return_url}
    task_id
} {
    # Returns delete or reactivate links for dropped tasks
    # and drop link for active tasks

    # Rule: ne. root_id
    ns_log notice "drop_option proc helper $return_url"

    if {[string equal $task_id $root_id] } {
	set option "<font color=gray>Drop</font>"
    } else {
	set option "
	<a href=task-drop?[export_vars -override {{v_task_id "$task_id"} \
		{action_type $link_type} {return_url "$return_url"}} \
		{v_task_id action_type return_url}]>[string totitle $link_type]</a> \n"
    }
    return $option
}

ad_proc -private drop_option {
    {-root_id ""}
    {-return_url}
    {-page_args ""}
    dropped_p
    task_id
} {
    switch $dropped_p {
	"t" {
	    set option_links  "
	    [drop_option_helper -root_id $root_id -link_type delete \
		    -return_url "$return_url" $task_id] |
	    [drop_option_helper -root_id $root_id -link_type reactivate \
		    -return_url "$return_url" $task_id]"
	} "f" {
	    set option_links  "
	    [drop_option_helper -root_id $root_id -link_type drop \
		    -return_url "$return_url" $task_id] "
	} default {
	    set option_links "error: invalid option"
	}
    }
    return $option_links
}

ad_proc -private zoom_option {
    {-root_id}
    task_id 
} {
    
    # We override the action_type setting because it doesn't make sense, and
    # could potentially cause things to break

    if {![string equal $root_id $task_id]} {
	return "
	(<a href=index?[append_anchor -anchor $task_id [export_vars -override \
		{{root_id $task_id} {action_type "default"}} \
		{root_id}]]>zoom in</a>)"
    } else {
	return ""
    }
}

ad_proc -private case_suspend_link {
    return_url
    state
    case_id
} {
    set case_suspend_link ""
    if { [string equal $state "suspended"] || [string equal $state "canceled"] } {
        return "
	<a href=state-change?[export_vars {{case_id $case_id} {action resume} \
		{return_url $return_url}}]>Resume</a>
	"
    }
    if {[string equal $state "active"]} {
	return  "
	<a href=state-change?[export_vars {{case_id $case_id} {action suspend} \
		{return_url $return_url}}]>Suspend</a>
	"
    }
}

# Generates links that effects workflow: notify, complete, suspend, resume

ad_proc -private workflow_link {
    {-wf_url}
    {-party_id ""}
    {-wf_task_id}
    {-case_id}
    {-wt_state ""}
    {-started_date ""}
    {-mine_p "f"}
    {-holding_user_p ""}
    {-return_url "index"}
    {-admin_p "f"}
    case_state
    task_id
} {
     
    # This proc generates the links to alter states for workflow
    # the possibilities are:
    # needs assignment
    # confirm (wf_case start) only assigned users
    # complete - only holding users
    # revert (wf_task cancel) holding user/admin
    # suspend (wf_case cancel) admin
    # resume (wf_case resume)  admin

    # party_id indicates if the task had been assigned, mine_p indicates if the task is mine.

    set workflow_link ""

    # adding the anchor
    set return_url [append_anchor -anchor $task_id $return_url]

    # Possible states are created, active, suspended, cancelled, finished
    if {[empty_string_p $party_id]} {
	set  workflow_link "<em>Needs assignment</em>"
    } else { 

	# created, active, canceled, suspended, finished
	# enabled, started, canceled, finished, overridden, 
	set case_suspend_link "[case_suspend_link $return_url $case_state $case_id]"		    

	# When the case is active, we look at the wf_task state
	switch $case_state {
	    created {

		# Start the case

		set workflow_link "
		<a href=workflow-start?[append_anchor -anchor $task_id [export_vars -url {{v_task_id $task_id} \
			{return_url $return_url}}]]>Notify</a>"
	    } active {
		
		if {[string equal $wt_state "started"]} {

		    # Only holding users can complete a task
		    # you also get the cancel link which lets go of the holding user
		    
		    set task_cancel_link "		    
		    <a href=task-action?[export_vars -url \
			    -override {{wf_task_id $wf_task_id} \
			    {return_url $return_url} \
			    {action cancel}} \
			    {wf_task_id return_url action}]>Revert</a>"

		    set complete_link "
		    <a href=$wf_url/task?[export_vars -url \
				-override {{task_id $wf_task_id} \
				{return_url $return_url}} \
				{task_id return_url }]>Complete</a>"

		    if {$holding_user_p} {
			set workflow_link "
			$complete_link |
			$task_cancel_link"
		    } else {
			set workflow_link "Complete"
		    }

		    append workflow_link "
		    <font size=-1>
		    <br>* Started on: $started_date
		    </font>
		    "
		} else {
		    set start_link "
		    <a href=task-action?[export_vars -url \
			    -override {{wf_task_id $wf_task_id} \
			    {return_url $return_url} \
			    {action start}} \
			    {wf_task_id return_url action}]>Start</a>"
		    
		    if {$mine_p} {
			#  We use our own page to start a task
			set workflow_link "$start_link"
		    } else {
			set workflow_link "Start"
		    }
		}
		
		# Only admins can suspend tasks
		if {$admin_p} {
		    append workflow_link "<br>($case_suspend_link)"
		}

	    } canceled {
		set workflow_link "$case_suspend_link"		    
		# Possible action is resume
	    } suspended {
		# Possible action is resume
		set workflow_link "$case_suspend_link"		    
	    } finished {
		set workflow_link "Finished"
	    } default {
		set workflow_link [string totitle $case_state]
	    }
	}
    }
    return $workflow_link
}

# Returns the possible actions given a state (cancel resume suspend)


ad_proc -private move_option {
    {-root_id ""} 
    {-one_level_two ""}
    {-page_args ""}
    {-v_task_id ""}
    dropped_p
    task_id
} {
    # Rule: ne. one_level_two, ne root, ne v_task_id
    # Tasks cannot move between states
 
    set move_link ""

    if {[string equal $task_id $root_id] || \
	    [string equal $one_level_two $task_id] || \
	    [string equal $v_task_id $task_id]  } {
	set move_link "<font color=gray>Move</font>"
    } else {
	# Everything is ok
	set move_link "
	<a href=index?[append_anchor -anchor $task_id \
		[export_vars -override {{v_task_id "$task_id"} \
		{action_type "move"}} {v_task_id action_type} ]]>Move</a>"
    }
    return $move_link
}

ad_proc -private ae_option {
    {-link_type "edit"}
    {-v_parent_id ""} 
    {-page_args ""}
    {-parent_depth ""}
    task_id
} {
    # Returns add and edit links for active tasks and only edit links
    # for dropped tasks

    # Rule not on task that is spawining (ne. v_parent_id)

    if {[string equal $v_parent_id $task_id]} {
	set option "<font color=gray>[string totitle $link_type]</font>\n"
    } else {
	switch $link_type {
	    edit {	
		set option  "<a href=index?[append_anchor -anchor $task_id [export_vars { \
			{action_type "ae"} \
			{v_task_id "$task_id"}}]]>[string totitle $link_type]</a>\n"
	    } add {
		set option  "
		<a href=index?[append_anchor -anchor $task_id [export_vars -override { \
			{action_type "ae"} \
			{v_parent_id "$task_id"} \
			{parent_depth $parent_depth} \
		    } {action_type v_parent_id parent_depth} \
			]]>[string totitle $link_type]</a>\n"
	    }
	}
    }
    return $option
}

ad_proc -private action_options {
    {-one_level_two}
    {-root_id}
    {-v_task_id}
    {-v_parent_id}
    {-parent_depth ""}
    {-page_args ""}
    dropped_p
    task_id
} {
    set action_options ""

    if {[string equal $dropped_p "t"]} {
	# edit
	append action_options "
	[ae_option -link_type edit -page_args $page_args $task_id] 
	"
    } else {
	# edit add drop move
	append action_options "
	[ae_option -link_type edit -page_args $page_args $task_id] | 
	[ae_option -link_type add -v_parent_id $v_parent_id -page_args $page_args \
		-parent_depth $parent_depth $task_id] | 
	[move_option -root_id $root_id -one_level_two $one_level_two \
		-page_args $page_args \
		-v_task_id $v_task_id \
		 $dropped_p $task_id] 
	"
    }
    # Generates the appropriate option list based on action_type
    return $action_options
}
 
ad_proc -private assignment_option {
    {-assigning_task ""}
    {-state ""}
    task_id
} {
    # Rule: ne. v_task_id
    set current_url [ad_conn url]

    if {[string equal $assigning_task $task_id] || \
	    [string equal $state "finished"]  } {
	return "&nbsp;"
    } else {
	return "
	<a href=$current_url?[append_anchor -anchor $task_id \
		[export_vars -override {{v_task_id $task_id} \ 
	{action_type assign}} \
		{v_task_id action_type}]]>Assign</a>"
    }
}

ad_proc -private move_target {
    {-moving_task "" }
    {-immediate_parent "" }
    {-decendent_p ""}
    {-page_args ""}
    {-parent_depth ""}
    {-level ""}
    dropped_p
    task_id 
} {
    # Rule ne. moving task branch.
    # Must be within same state

    if {[string equal $parent_depth $level]} {
	incr level
    }

    set return_url "index?[append_anchor -anchor $task_id [export_vars {level}]]"

    if {![string equal $dropped_p "dropped"] && [string equal $decendent_p "f"] && \
	![string equal $immediate_parent $task_id]} {
	set target_link "
	<font size=-1>(
	<a href=task-move?[export_vars {{v_task_id $moving_task} \
		{v_parent_id $task_id} {return_url $return_url}}]>
	Move here</a>)
	</font>"
    } else {
	set target_link ""
    }
    return $target_link
}


ad_proc -private get_ancestors {
    {-var_to_export} 
    {-current_task}
    v_task_id
} {
    db_multirow ancestors ancestor_query {
	select t.task_id, h.depth, t.name, t.description,
	'' as name_link
	from pt_tasks t, 
	    ( select task_id, level as depth
	      from pt_tasks
              start with task_id = :v_task_id
	      connect by task_id = prior parent_id ) h
	where h.task_id = t.task_id
	order by depth desc
    } {
	
	if {[string equal $v_task_id $task_id]} {
	    set name_link [subst $current_task]
	} else {
	    set name_link "<a href=[ad_conn url]?[export_vars [list [list $var_to_export $task_id]]]>$name</a> \n"
	    
	}
    }

}

ad_proc -private action_type_pretty {
    action_type
} {
    array set pretty {
	ae "Add/Edit"
	move "Move"
	assign "Assign"
	admin "Admin"
	default "Default"
    }
    return $pretty($action_type)
}

ad_proc priority_option {
    form_name
} {
    set package_id [ad_conn package_id]
    set priority_options [db_list_of_lists priority_option_select {  
	select name, priority_id 
	from pt_priorities
	where package_id = :package_id
	order by value desc
    }]

    template::element create $form_name priority \
	    -datatype integer \
	    -label "Priority" \
	    -widget select \
	    -options "{{ unknown } {} }
	$priority_options"
	
}

ad_proc -private task_ae_form {
    {-parent_depth ""}
    {-level ""}
    {-task_states ""}
    {-v_task_id ""}
    {-v_parent_id ""}
    {-leaf_p "t"}
    form_name
    return_url
} {

    template::element create $form_name v_task_id \
	    -widget hidden
    
    template::element create $form_name return_url \
	    -widget hidden
    
    template::element create $form_name v_parent_id \
	    -widget hidden

    template::element create $form_name name \
	    -widget text \
	    -label "Name"
    
    template::element create $form_name description \
	    -widget textarea \
	    -label "Description" \
	    -html {columns 15 rows 5 wrap virtual}

    template::element create $form_name html_p \
	    -label "&nbsp" \
	    -datatype string \
	    -widget select \
	    -options {
	{HTML t}
	{"Plain text" f}
    }

    priority_option $form_name

    if {$leaf_p} {
    template::element create $form_name estimated_days \
	    -label "Estimated Days" \
	    -widget text \
	    -html {size 3}

    template::element create $form_name deadline \
	    -datatype date \
	    -label "Deadline" \
	    -format YYYY/MON/DD \
	    -year_interval { 2001 2005 1} \
	    -widget date
    }

    if {[template::form is_request $form_name]} {
	if {[exists_and_not_null v_parent_id]} {
	    # Adding: set the object_id for the new task
	    # we want to consider increasing the level...

	    if {[exists_and_not_null parent_depth]} {
		if {[string equal $parent_depth  $level]} {
		    incr level
		}
	    }

	    # Since when you are adding a task, you are, by
	    # definition, adding an incomplete task, if you are in
	    # finished filter, we pop you out of it.
	    if {[string equal $task_states "finished"]} {
		set task_states "active"
	    }
	    
	    template::element set_properties $form_name v_task_id \
		    -value [db_nextval "acs_object_id_seq"]
	    
	    template::element set_properties $form_name v_parent_id \
		    -value $v_parent_id

	    template::element set_properties $form_name return_url \
		    -value "index?[append_anchor -anchor $v_parent_id \
		    [export_vars \
		    {{action_type default} level task_states}]]"
	    
	} else {
	    # Editing: set the values
	    db_0or1row task_edit_select {
		select name, 
		description, 
		estimated_days, 
		to_char(deadline, 'YYYY MM DD HH24 MI SS') as deadline,
		parent_id, 
		leaf_p,
		html_p,
		priority
		from pt_tasks
		where task_id = :v_task_id 
	    }
	    
	    set deadline [template::util::date acquire sql_date $deadline]

	    # now we set the values
	    template::element set_properties $form_name v_task_id \
		    -value $v_task_id
	    
	    template::element set_properties $form_name v_parent_id \
		    -value $parent_id
	    
	    template::element set_properties $form_name name \
		    -value $name

	    template::element set_properties $form_name description \
		    -value $description

	    if {$leaf_p} {
		template::element set_properties $form_name estimated_days \
		    -value $estimated_days
		
		template::element set_properties $form_name deadline \
			-widget date \
			-format YYYY/MON/DD \
			-value $deadline 
	    }

	    template::element set_properties $form_name return_url \
		    -value "$return_url"

	    template::element set_properties $form_name html_p \
		    -value $html_p

	    template::element set_properties $form_name priority \
		    -value $priority
	}
    }
}









 