# /packages/project-tracker/www/index.tcl

ad_page_contract {
    
    Main page for project tracker. 

    @creation-date Feb 26 2001
    @author Nobuko Asakai
    @cvs-id $Id: index.tcl,v 1.4 2001/04/17 20:19:07 nasakai Exp $

    @param v_task_id The task that is being acted upon for the various
    action_types. We don't have the pt_task_id filter on this because
    it could be a newly fetched id for the form.

    @param action_type Diffrent data manipulations for task
    attributes.Possible types are "admin" "default" "ae"
    "move" "assign"

    These parameters (detail, task_states, view_type, level, root_id)
    are the dimensional sliders for the index
    page. The default values are set by ad_get_client_property to make
    them persistent.
    @param detail Hide and unhiding task descriptions
    @param task_states Hide and unhiding task_states tasks, finished
    filter is not allowed when you are adding tasks
    @view_type Different columns are shown for different states
     permissions one of read-only, default (write), and admin
    @param level How deep you want to see in the task tree. The level
    is started at 2 to speed up loading. Level is increased when moving or adding
    @param root_id  Root node to start the downward traversal from

    @param user_search the keyword for searching users 
    @param parent_depth depth of the parent that is being acted
    on. Only useful for move and add options
} {
    {v_task_id:integer ""}
    {v_parent_id:integer,pt_task_id ""}
    {action_type:trim "default"}
    {detail "[ad_get_client_property -default all [ad_conn package_id] detail]"}
    {task_states "[ad_get_client_property -default active [ad_conn package_id] task_states]"}
    {level "[ad_get_client_property -default 2 [ad_conn package_id] level]"}
    {root_id:integer,pt_task_id "[ad_get_client_property [ad_conn package_id] root_id]"}
    {view_type "[ad_get_client_property [ad_conn package_id] view_type]"}
    {deadline_slider "[ad_get_client_property -default all [ad_conn package_id] deadline_slider]"}
    {action_type "default"}
    {search_keyword:nohtml ""}
    {parent_depth:integer,trim ""}
} -properties {
    task_list
    action_type
    dimensional_slider
    ancestors
    total
    total_complete
} -validate {
    action_type_check -requires action_type {
	if {![string equal $action_type "ae"] && \
		![string equal $action_type "admin"] && \
		![string equal $action_type "default"] && \
		![string equal $action_type "assign"] && \
		![string equal $action_type "move"]} {
	    ad_complain "$action_type is not a valid action_type, must be one of ae, admin, assign, move, default"
	}
    }

    action_type_and_id_existence_check -requires action_type {
	# Some of the types require a task-id, so we insist that it's prensent
	if {[string equal $action_type "ae"]} {
	    if {![exists_and_not_null v_task_id] && ![exists_and_not_null v_parent_id]} {
		set action_type "default"
	    }
	}
	if {[string equal $action_type "assign"] || [string equal $action_type "move"] } {
	    if {![exists_and_not_null v_task_id]} {
		set action_type "default"
	    }
	}
    }
    
    maybe_get_root {
	if {![exists_and_not_null root_id]} {
	    set root_id [pt_get_root]
	}
    }
    
    set_dimensional_properties {
	# Update the cache
	# It would be good if we could only do this when the value is passed in.
	# For now we set persistent to f so we do not incure a db hit

	set package_id [ad_conn package_id]
	ad_set_client_property -persistent f $package_id detail $detail
	ad_set_client_property -persistent f $package_id level $level
	ad_set_client_property -persistent f $package_id root_id $root_id
	ad_set_client_property -persistent f $package_id task_states $task_states
	ad_set_client_property -persistent f $package_id view_type $view_type
	ad_set_client_property -persistent f $package_id deadline_slider $deadline_slider
    }
}

# Redirect if there is no root task
if {[empty_string_p $root_id]} {
    # redirect to the initialization page
    ad_returnredirect "admin/initialize"
    return
} 

# Master template properties

# Pulling out the project information
set package_id [ad_conn package_id]

db_1row project-info {
    select max(level) as max_level,
           pt_project.one_level_two (:root_id) as one_level_two, 
           pt_util.workflow_url as wf_url,
           apm_package.name(:package_id) as package_name
    from pt_tasks
   start with task_id = :root_id
   connect  by parent_id = prior task_id
} 

set page_title "$package_name"
set context_bar {}

# Permissions
set user_id [ad_conn user_id]
set read_p [ad_permission_p -user_id $user_id $package_id "read"]
set admin_p [ad_permission_p -user_id $user_id $package_id "admin"]
set write_p [ad_permission_p -user_id $user_id $package_id "write"]

# view types
# We need to switch out the dimensional slider for views for people
# with only read permissions

if {$read_p && !$admin_p && !$write_p} {
    # User only has read permission
    set view_type "read-only"
    set view_dimensions [list view_type "Views" read-only [list [list read-only {Read Only} {}]]]
} elseif { $write_p || $admin_p} {
    set view_dimensions [list view_type "Views" write [list [list read-only "Read-Only" [list ]] [list write "Write" [list ]]]]
}

# We want to say, if you aren't logged in, but you have permission,
# you can read it, but if you request anything else, you'll be
# redirected
if {![string equal $action_type "default"]} {
    ad_maybe_redirect_for_registration
}

# We traverse the tree upwards to get the ancestors of the current root task

get_ancestors -var_to_export root_id -current_task {$name} $root_id

# We use the list format instead of the curly braces because we want
# the default values to be sustitued

set dimensional_list [list ]

set task_state_dimension [list task_states {Task&nbsp;states} $task_states {
 	{all "All" { where "1 = 1" }}
  	{active "Active only" {where "dropped_p = 'f'" }}
	{incomplete "Incomplete" {where "(wc.state <> 'finished' or wc.state IS NULL)
	and dropped_p = 'f'"}} 
	{finished "Finished" {where "
	(wc.state = 'finished' or (
	wc.state IS NULL and leaf_p = 'f'))
	and dropped_p = 'f'"}} 
    }]

set detail_dimension [list detail {Detail} $detail {
 	{all "All" {}}
  	{summary "Summary" {}}
}]

set deadline_dimension [list deadline_slider {Deadline} "$deadline_slider" {
    {all "All" {}}
    {today "Today" { where "(trunc(t.deadline) <= trunc(sysdate) or t.deadline is NULL)"}}
    {one_week "One week" {where "(trunc(t.deadline) <= trunc(sysdate +7) 
    or t.deadline is NULL)" }}
}]

lappend dimensional_list $task_state_dimension
lappend dimensional_list $detail_dimension
lappend dimensional_list $deadline_dimension

# We make sure that level never exceeds max_level by more than one.
# need add one because when you are adding new tasks, and the level of
# the spawning task is the same as maxlevel, we need to increase it.

if {[expr {$level > $max_level}] } {
    set level $max_level
}

set level_demension [list level "Level" $level]

# The default for level is 2 to speed up loading..
# and i is started at 2 to prevent 1 from being a link

set level_dimension_items [list ]
for {set i 1} {$i <= $max_level} {incr i} {
    lappend level_dimension_items [list $i "$i" [list where "depth <= [subst $i]"]]
}

lappend level_demension $level_dimension_items
lappend dimensional_list $level_demension

lappend dimensional_list $view_dimensions

set dimensional_slider [ad_dimensional $dimensional_list index]

# Link to take user to extreme actions
set url [ad_conn url]

if {$admin_p && ![string equal $action_type admin]} {
    set drop_link "<a href=$url?[export_vars -url \
	    -override {{action_type "admin"}} view_type]>Extreme actions (admin)</a>"
} else {
    set drop_link ""
}

# Everything from here down can go to the task_list?
# need to pass dimensional_list

#  Return url  anchor is appended here
set return_url ${url}

if {[exists_and_not_null v_task_id]} {
    # We add a question mark to the plain url
    set return_url [append_anchor -anchor $v_task_id  $return_url?]
}

if {![string equal $view_type "read-only"]} {  
    # We only execute this query if the view is not read-only

    # These two queries should be merged into one
    # nasakai@arsdigita.com Mar 27, 2001

    db_1row totals_query { 
	select nvl(sum(estimated_days),0) as estimated_days,
	       nvl(sum(actual_days), 0) as actual_days,
	       count(task_id) as task_count
	from pt_tasks 
	where leaf_p = 't' 
	and   dropped_p = 'f' 
	start with task_id =:root_id
	connect by parent_id = prior task_id 
    } -column_array total

    db_1row completed_total {
	select count(*) as finished_count,
	       nvl( sum ( estimated_days),0) as finished_estimate
	from pt_tasks pt, wf_cases wc, (
	     select task_id
	     from pt_tasks
	     where leaf_p = 't'
	     and   dropped_p = 'f'
	     start with task_id =:root_id
	     connect by parent_id = prior task_id ) h
	where  h.task_id = pt.task_id
	and    wc.state = 'finished'
	and    pt.task_id = wc.object_id(+)
    } -column_array {total_complete}


    set incomplete_estimate [expr { $total(estimated_days) - $total_complete(finished_estimate)}]

    set incomplete_count [expr { $total(task_count) - $total_complete(finished_count)}]

    set wf_link "\n<a href=$wf_url>My workflow task list\n"

} else {
    set wf_link "&nbsp;"
}

set immediate_parent ""
set table_color "#cccff"

set comma_extra_col ", '#FFFFFF' as bgcolor"
set comma_extra_table ""
set and_extra_join ""

set form_name "task_ae"

form::create $form_name \
	-action "task-ae"

# One option is to move this to a proc so it can be reused for task-view

switch $action_type {
    ae {
	set return_url "index?[append_anchor -anchor $v_task_id \
		    [export_vars \
		    {{action_type default} level task_states}]]"

	task_ae_form -level $level \
		-task_states $task_states \
		-v_task_id $v_task_id \
		-v_parent_id $v_parent_id \
		-parent_depth $parent_depth $form_name $return_url 
		
    } move {
	set immediate_parent [db_string get_immediate_parent "
	select parent_id from pt_tasks where task_id = :v_task_id"]

	set comma_extra_col "
	, '' as move_target , 
	decode(decendent.task_id, NULL, '#FFFFFF', '#CCFFFF') as bgcolor,
	decode(decendent.task_id,NULL, 'f', 't') as decendent_p "

	set comma_extra_table "
	, (select task_id 
	from pt_tasks
	start with task_id = :v_task_id
	connect by parent_id = prior task_id) decendent
	"
	set and_extra_join "and t.task_id = decendent.task_id(+)"

    } assign {
	# Do nothing
    } default {
	    set default_view "<font color=gray>Default</font>"
    }
}

set task_list_query "
select t.name, 
       t.task_id,
       t.description, 
       decode(t.estimated_days, NULL, '&nbsp;', t.estimated_days) as estimated_days,
       decode(t.deadline,NULL, '&nbsp;', t.deadline) as deadline,
       decode(t.actual_days,NULL, '&nbsp;', t.actual_days) as actual_days,
       nvl(to_char(t.deadline, 'YYYY/Mon/DD'), '&nbsp;') as deadline,
       h.depth, 
       h.task_num,
       dropped_p,
       html_p,
       leaf_p,
       nvl(pp.name, '&nbsp;') as priority,
       pp.value as priority_value,
       wt.state as wt_state,
       wt.holding_user,
       to_char(wt.started_date, 'Mon/DD/YYYY') as started_date,
       wc.state,
       wc.case_id,
       nvl(wt.task_id,'') as wf_task_id,
       wca.party_id,
       wca.party_id as assignee_id,
       acs_object.name(wca.party_id) as assignment_name,
       p.email,
       decode (holding_user, :user_id, 't', 'f') as holding_user_p,
       (trunc(sysdate) - trunc(deadline)) as days_late,
       nvl2(mine.case_id, 't', 'f') as mine_p,
       gc.comment_count,
       decode(dropped_p, 't', 'gray', 'black') as fontcolor,
       '' as action_links,
       '' as todo_link,
       '' as admin_options,
       '' as task_view_link,
       '' as name_link,
       '' as assignment_option,
       '' as zoom_option
       $comma_extra_col
from pt_tasks t, pt_priorities pp,
     (select /*+INDEX(pt_task,test_index)*/ task_id, level as depth, rownum as task_num
      from pt_tasks
      start with task_id = :root_id
      connect by parent_id = prior task_id) h,
     (select count(content_item.get_live_revision(comment_id)) as comment_count, 
              object_id
      from general_comments 
     group by object_id) gc,
     wf_cases wc, 
     (select task_id, holding_user, started_date, state, case_id from wf_tasks 
      where (state = 'enabled' or state = 'started' or state = 'finished')) wt, 
     wf_case_assignments wca, 
     (select case_id 
      from pt_case_assignments
      where user_id = :user_id
     ) mine,
     parties p $comma_extra_table
where h.task_id = t.task_id
and t.task_id = gc.object_id(+)
and t.task_id = wc.object_id(+)
and wc.case_id = wt.case_id(+)
and wc.case_id = wca.case_id(+)
and wc.case_id = mine.case_id(+)
and wca.party_id = p.party_id(+)
and t.priority = pp.priority_id(+)
$and_extra_join
[ad_dimensional_sql $dimensional_list where and]
order by task_num
"

db_multirow task_list task_select $task_list_query {
    
    # We don't generate any links if we are in read-only except for
    # zoom

    # General edit, add, move link

    # Link to task-view
    if {[string equal $view_type read-only]} {
	set name_link $name

	if {[string equal $leaf_p "t"] && [string equal $dropped_p "f"]} {
	    # Only for active leaves
	    if {[string equal $state "finished"]} {
		set todo_link "Finished"
	    } else {
		set todo_link "Incomplete"
	    }
	} elseif { [string equal $dropped_p "t"]} {
	    set todo_link "Dropped"
	} else {
	    # These are parent tasks that are not dropped
	    set todo_link " - "
	    set zoom_option [zoom_option -root_id $root_id $task_id]
	}

	set action_links ""
    } else {
	set name_link "<a href=task-view?[export_vars -url {{v_task_id $task_id}}]>$name</a>"

	# Links for assignment and To Do column

	if {[string equal $leaf_p "t"] && [string equal $dropped_p "f"]} {
	    # Only for active leaves
	    set assignment_option [assignment_option \
		    -state $state \
		    $task_id]

	    set todo_link  [workflow_link -wf_url $wf_url \
		    -party_id $party_id \
		    -wf_task_id $wf_task_id \
		    -case_id $case_id \
		    -wt_state $wt_state \
		    -started_date $started_date \
		    -return_url $url? \
		    -mine_p $mine_p \
		    -holding_user_p $holding_user_p \
		    -admin_p $admin_p \
		    $state $task_id]
	    
	    set zoom_option ""
	} elseif { [string equal $dropped_p "t"]} {
	    set todo_link "Dropped"
	    set assignment_option "&nbsp;"
	} else {
	    # These are parent tasks that are not dropped
	    set todo_link " - "
	    set assignment_option "&nbsp;"

	    set zoom_option [zoom_option -root_id $root_id $task_id]
	}

	# Links that you get when you are in move or admin mode
	switch $action_type {
	    "move" {
		set action_links [move_target -moving_task $v_task_id \
			-immediate_parent $immediate_parent \
			-decendent_p $decendent_p \
			-parent_depth $depth \
			-level $level \
			$dropped_p $task_id]
	    } "admin" {
		set action_links [drop_option -root_id $root_id \
			-return_url $return_url \
			$dropped_p $task_id]	    
	    } default {
		set action_links  [action_options \
			-one_level_two $one_level_two \
			-root_id $root_id \
			-v_parent_id $v_parent_id \
			-v_task_id $v_task_id \
			-parent_depth $depth \
			$dropped_p \
			$task_id ]
	    }
	}
    }
}


set test [ns_urlencode #]

ad_return_template index


























